@@ -1,25 +1,19 @@
 """Logging Utilities."""
-from __future__ import absolute_import, unicode_literals
 
 import logging
 import numbers
 import os
 import sys
-
 from logging.handlers import WatchedFileHandler
 
-from .five import string_t
 from .utils.encoding import safe_repr, safe_str
 from .utils.functional import maybe_evaluate
 from .utils.objects import cached_property
 
 __all__ = ('LogMixin', 'LOG_LEVELS', 'get_loglevel', 'setup_logging')
 
-try:
-    LOG_LEVELS = dict(logging._nameToLevel)
-    LOG_LEVELS.update(logging._levelToName)
-except AttributeError:
-    LOG_LEVELS = dict(logging._levelNames)
+LOG_LEVELS = dict(logging._nameToLevel)
+LOG_LEVELS.update(logging._levelToName)
 LOG_LEVELS.setdefault('FATAL', logging.FATAL)
 LOG_LEVELS.setdefault(logging.FATAL, 'FATAL')
 DISABLE_TRACEBACKS = os.environ.get('DISABLE_TRACEBACKS')
@@ -27,7 +21,7 @@ DISABLE_TRACEBACKS = os.environ.get('DISABLE_TRACEBACKS')
 
 def get_logger(logger):
     """Get logger by name."""
-    if isinstance(logger, string_t):
+    if isinstance(logger, str):
         logger = logging.getLogger(logger)
     if not logger.handlers:
         logger.addHandler(logging.NullHandler())
@@ -36,7 +30,7 @@ def get_logger(logger):
 
 def get_loglevel(level):
     """Get loglevel by name."""
-    if isinstance(level, string_t):
+    if isinstance(level, str):
         return LOG_LEVELS[level]
     return level
 
@@ -54,7 +48,7 @@ def safeify_format(fmt, args, filters=None):
         yield filt(args[index]) if filt else args[index]
 
 
-class LogMixin(object):
+class LogMixin:
     """Mixin that adds severity methods to any class."""
 
     def debug(self, *args, **kwargs):
@@ -75,14 +69,14 @@ class LogMixin(object):
         return self.log(logging.CRITICAL, *args, **kwargs)
 
     def annotate(self, text):
-        return '%s - %s' % (self.logger_name, text)
+        return f'{self.logger_name} - {text}'
 
     def log(self, severity, *args, **kwargs):
         if DISABLE_TRACEBACKS:
             kwargs.pop('exc_info', None)
         if self.logger.isEnabledFor(severity):
             log = self.logger.log
-            if len(args) > 1 and isinstance(args[0], string_t):
+            if len(args) > 1 and isinstance(args[0], str):
                 expand = [maybe_evaluate(arg) for arg in args[1:]]
                 return log(severity,
                            self.annotate(args[0].replace('%r', '%s')),
@@ -121,7 +115,7 @@ class Log(LogMixin):
     def get_logger(self):
         if self._logger:
             return self._logger
-        return LogMixin.get_logger(self)
+        return super().get_logger()
 
     @property
     def logger_name(self):
