@@ -3,6 +3,7 @@ from contextlib import closing
 from time import sleep
 
 import pytest
+
 import kombu
 
 
@@ -143,19 +144,19 @@ class BaseExchangeTypes:
         with consumer:
             connection.drain_events(timeout=1)
 
-    def _publish(self, channel, exchange, queues, routing_key=None):
+    def _publish(self, channel, exchange, queues=None, routing_key=None):
         producer = kombu.Producer(channel, exchange=exchange)
         if routing_key:
             producer.publish(
                 {'hello': 'world'},
-                declare=list(queues),
+                declare=list(queues) if queues else None,
                 serializer='pickle',
                 routing_key=routing_key
             )
         else:
             producer.publish(
                 {'hello': 'world'},
-                declare=list(queues),
+                declare=list(queues) if queues else None,
                 serializer='pickle'
             )
 
@@ -180,6 +181,13 @@ class BaseExchangeTypes:
                 # direct2 queue should not have data
                 with pytest.raises(socket.timeout):
                     self._consume(conn, test_queue2)
+                # test that publishing using key which is not used results in
+                # discarted message.
+                self._publish(channel, ex, [test_queue1, test_queue2], 'd3')
+                with pytest.raises(socket.timeout):
+                    self._consume(conn, test_queue1)
+                with pytest.raises(socket.timeout):
+                    self._consume(conn, test_queue2)
 
     def test_fanout(self, connection):
         ex = kombu.Exchange('test_fanout', type='fanout')
@@ -212,6 +220,15 @@ class BaseExchangeTypes:
                     # topic3 queue should not have data
                     self._consume(conn, test_queue3)
 
+    def test_publish_empty_exchange(self, connection):
+        ex = kombu.Exchange('test_empty_exchange', type='topic')
+        with connection as conn:
+            with conn.channel() as channel:
+                self._publish(
+                    channel, ex,
+                    routing_key='t.1'
+                )
+
 
 class BaseTimeToLive:
     def test_publish_consume(self, connection):
