@@ -1,20 +1,27 @@
 import os
-
-import pytest
-import kombu
 from time import sleep
 
-from .common import (
-    BasicFunctionality, BaseExchangeTypes, BasePriority
-)
+import pytest
+import redis
+
+import kombu
+
+from .common import BaseExchangeTypes, BasePriority, BasicFunctionality
 
 
 def get_connection(
-        hostname, port, vhost):
-    return kombu.Connection(f'redis://{hostname}:{port}')
+        hostname, port, vhost, user_name=None, password=None,
+        transport_options=None):
+
+    credentials = f'{user_name}:{password}@' if user_name else ''
+
+    return kombu.Connection(
+        f'redis://{credentials}{hostname}:{port}',
+        transport_options=transport_options
+    )
 
 
-@pytest.fixture()
+@pytest.fixture(params=[None, {'global_keyprefix': '_prefixed_'}])
 def connection(request):
     # this fixture yields plain connections to broker and TLS encrypted
     return get_connection(
@@ -23,6 +30,7 @@ def connection(request):
         vhost=getattr(
             request.config, "slaveinput", {}
         ).get("slaveid", None),
+        transport_options=request.param
     )
 
 
@@ -31,6 +39,19 @@ def invalid_connection():
     return kombu.Connection('redis://localhost:12345')
 
 
+@pytest.mark.env('redis')
+def test_failed_credentials():
+    """Tests denied connection when wrong credentials were provided"""
+    with pytest.raises(redis.exceptions.ResponseError):
+        get_connection(
+            hostname=os.environ.get('REDIS_HOST', 'localhost'),
+            port=os.environ.get('REDIS_6379_TCP', '6379'),
+            vhost=None,
+            user_name='wrong_redis_user',
+            password='wrong_redis_password'
+        ).connect()
+
+
 @pytest.mark.env('redis')
 @pytest.mark.flaky(reruns=5, reruns_delay=2)
 class test_RedisBasicFunctionality(BasicFunctionality):
