@@ -1,17 +1,53 @@
-import pytest
 import socket
 import types
-
 from collections import defaultdict
 from itertools import count
-from queue import Empty, Queue as _Queue
-from case import ANY, ContextMock, Mock, call, mock, skip, patch
+from queue import Empty
+from queue import Queue as _Queue
+from unittest.mock import ANY, Mock, call, patch
 
-from kombu import Connection, Exchange, Queue, Consumer, Producer
-from kombu.exceptions import InconsistencyError, VersionMismatch
+import pytest
+
+from kombu import Connection, Consumer, Exchange, Producer, Queue
+from kombu.exceptions import VersionMismatch
 from kombu.transport import virtual
 from kombu.utils import eventio  # patch poll
 from kombu.utils.json import dumps
+from t.mocks import ContextMock
+
+
+def _redis_modules():
+
+    class ConnectionError(Exception):
+        pass
+
+    class AuthenticationError(Exception):
+        pass
+
+    class InvalidData(Exception):
+        pass
+
+    class InvalidResponse(Exception):
+        pass
+
+    class ResponseError(Exception):
+        pass
+
+    exceptions = types.ModuleType('redis.exceptions')
+    exceptions.ConnectionError = ConnectionError
+    exceptions.AuthenticationError = AuthenticationError
+    exceptions.InvalidData = InvalidData
+    exceptions.InvalidResponse = InvalidResponse
+    exceptions.ResponseError = ResponseError
+
+    class Redis:
+        pass
+
+    myredis = types.ModuleType('redis')
+    myredis.exceptions = exceptions
+    myredis.Redis = Redis
+
+    return myredis, exceptions
 
 
 class _poll(eventio._select):
@@ -29,6 +65,9 @@ class _poll(eventio._select):
 
 
 eventio.poll = _poll
+
+pytest.importorskip('redis')
+
 # must import after poller patch, pep8 complains
 from kombu.transport import redis  # noqa
 
@@ -236,7 +275,6 @@ class Transport(redis.Transport):
         return ((KeyError,), (IndexError,))
 
 
-@skip.unless_module('redis')
 class test_Channel:
 
     def setup(self):
@@ -310,6 +348,7 @@ class test_Channel:
 
     def test_get_redis_ConnectionError(self):
         from redis.exceptions import ConnectionError
+
         from kombu.transport.redis import get_redis_ConnectionError
         connection_error = get_redis_ConnectionError()
         assert connection_error == ConnectionError
@@ -537,6 +576,12 @@ class test_Channel:
 
         assert self.channel._receive_one(self.channel.subclient) is None
 
+    def test_receive_invalid_response_type(self):
+        s = self.channel.subclient = Mock()
+        for resp in ['foo', None]:
+            s.parse_response.return_value = resp
+            assert self.channel._receive_one(self.channel.subclient) is None
+
     def test_receive_connection_has_gone(self):
         def _receive_one(c):
             c.connection = None
@@ -691,6 +736,18 @@ class test_Channel:
         self.channel.connection.client.hostname = 'george.vandelay.com'
         assert self.channel._connparams()['host'] == 'george.vandelay.com'
 
+    def test_connparams_username(self):
+        self.channel.connection.client.userid = 'kombu'
+        assert self.channel._connparams()['username'] == 'kombu'
+
+    def test_connparams_client_credentials(self):
+        self.channel.connection.client.hostname = \
+            'redis://foo:bar@127.0.0.1:6379/0'
+        connection_parameters = self.channel._connparams()
+
+        assert connection_parameters['username'] == 'foo'
+        assert connection_parameters['password'] == 'bar'
+
     def test_connparams_password_for_unix_socket(self):
         self.channel.connection.client.hostname = \
             'socket://:foo@/var/run/redis.sock'
@@ -729,7 +786,7 @@ class test_Channel:
     def test_get_client(self):
         import redis as R
         KombuRedis = redis.Channel._get_client(self.channel)
-        assert KombuRedis
+        assert isinstance(KombuRedis(), R.StrictRedis)
 
         Rv = getattr(R, 'VERSION', None)
         try:
@@ -740,6 +797,12 @@ class test_Channel:
             if Rv is not None:
                 R.VERSION = Rv
 
+    def test_get_prefixed_client(self):
+        from kombu.transport.redis import PrefixedStrictRedis
+        self.channel.global_keyprefix = "test_"
+        PrefixedRedis = redis.Channel._get_client(self.channel)
+        assert isinstance(PrefixedRedis(), PrefixedStrictRedis)
+
     def test_get_response_error(self):
         from redis.exceptions import ResponseError
         assert redis.Channel._get_response_error(self.channel) is ResponseError
@@ -844,13 +907,12 @@ class test_Channel:
             ('celery', '', 'celery'),
         ]
 
-        # ... then for some reason, the _kombu.binding.celery key gets lost
+        # Remove one last queue from exchange. After this call no queue
+        # is in bound to exchange.
         channel.client.srem(key)
 
-        # which raises a channel error so that the consumer/publisher
-        # can recover by redeclaring the required entities.
-        with pytest.raises(InconsistencyError):
-            self.channel.get_table('celery')
+        # get_table() should return empty list of queues
+        assert self.channel.get_table('celery') == []
 
     def test_socket_connection(self):
         with patch('kombu.transport.redis.Channel._create_client'):
@@ -909,8 +971,44 @@ class test_Channel:
                 ('celery', '', 'celery'),
             ]
 
+    @patch("redis.StrictRedis.execute_command")
+    def test_global_keyprefix(self, mock_execute_command):
+        from kombu.transport.redis import PrefixedStrictRedis
+
+        with Connection(transport=Transport) as conn:
+            client = PrefixedStrictRedis(global_keyprefix='foo_')
+
+            channel = conn.channel()
+            channel._create_client = Mock()
+            channel._create_client.return_value = client
+
+            body = {'hello': 'world'}
+            channel._put_fanout('exchange', body, '')
+            mock_execute_command.assert_called_with(
+                'PUBLISH',
+                'foo_/{db}.exchange',
+                dumps(body)
+            )
+
+    @patch("redis.StrictRedis.execute_command")
+    def test_global_keyprefix_queue_bind(self, mock_execute_command):
+        from kombu.transport.redis import PrefixedStrictRedis
+
+        with Connection(transport=Transport) as conn:
+            client = PrefixedStrictRedis(global_keyprefix='foo_')
+
+            channel = conn.channel()
+            channel._create_client = Mock()
+            channel._create_client.return_value = client
+
+            channel._queue_bind('default', '', None, 'queue')
+            mock_execute_command.assert_called_with(
+                'SADD',
+                'foo__kombu.binding.default',
+                '\x06\x16\x06\x16queue'
+            )
+
 
-@skip.unless_module('redis')
 class test_Redis:
 
     def setup(self):
@@ -921,8 +1019,8 @@ class test_Redis:
     def teardown(self):
         self.connection.close()
 
-    @mock.replace_module_value(redis.redis, 'VERSION', [3, 0, 0])
-    def test_publish__get_redispyv3(self):
+    @pytest.mark.replace_module_value(redis.redis, 'VERSION', [3, 0, 0])
+    def test_publish__get_redispyv3(self, replace_module_value):
         channel = self.connection.channel()
         producer = Producer(channel, self.exchange, routing_key='test_Redis')
         self.queue(channel).declare()
@@ -934,8 +1032,8 @@ class test_Redis:
         assert self.queue(channel).get() is None
         assert self.queue(channel).get() is None
 
-    @mock.replace_module_value(redis.redis, 'VERSION', [2, 5, 10])
-    def test_publish__get_redispyv2(self):
+    @pytest.mark.replace_module_value(redis.redis, 'VERSION', [2, 5, 10])
+    def test_publish__get_redispyv2(self, replace_module_value):
         channel = self.connection.channel()
         producer = Producer(channel, self.exchange, routing_key='test_Redis')
         self.queue(channel).declare()
@@ -1030,14 +1128,15 @@ class test_Redis:
             channel._get('does-not-exist')
         channel.close()
 
-    def test_get_client(self):
-        with mock.module_exists(*_redis_modules()):
-            conn = Connection(transport=Transport)
-            chan = conn.channel()
-            assert chan.Client
-            assert chan.ResponseError
-            assert conn.transport.connection_errors
-            assert conn.transport.channel_errors
+    @pytest.mark.ensured_modules(*_redis_modules())
+    def test_get_client(self, module_exists):
+        # with module_exists(*_redis_modules()):
+        conn = Connection(transport=Transport)
+        chan = conn.channel()
+        assert chan.Client
+        assert chan.ResponseError
+        assert conn.transport.connection_errors
+        assert conn.transport.channel_errors
 
     def test_check_at_least_we_try_to_connect_and_fail(self):
         import redis
@@ -1048,41 +1147,6 @@ class test_Redis:
             chan._size('some_queue')
 
 
-def _redis_modules():
-
-    class ConnectionError(Exception):
-        pass
-
-    class AuthenticationError(Exception):
-        pass
-
-    class InvalidData(Exception):
-        pass
-
-    class InvalidResponse(Exception):
-        pass
-
-    class ResponseError(Exception):
-        pass
-
-    exceptions = types.ModuleType('redis.exceptions')
-    exceptions.ConnectionError = ConnectionError
-    exceptions.AuthenticationError = AuthenticationError
-    exceptions.InvalidData = InvalidData
-    exceptions.InvalidResponse = InvalidResponse
-    exceptions.ResponseError = ResponseError
-
-    class Redis:
-        pass
-
-    myredis = types.ModuleType('redis')
-    myredis.exceptions = exceptions
-    myredis.Redis = Redis
-
-    return myredis, exceptions
-
-
-@skip.unless_module('redis')
 class test_MultiChannelPoller:
 
     def setup(self):
@@ -1130,25 +1194,6 @@ class test_MultiChannelPoller:
             num=chan1.unacked_restore_limit,
         )
 
-    def test_restore_visible_with_gevent(self):
-        with patch('kombu.transport.redis.time') as time:
-            with patch('kombu.transport.redis._detect_environment') as env:
-                timeout = 3600
-                time.return_value = timeout
-                env.return_value = 'gevent'
-                chan1 = Mock(name='chan1')
-                redis_ctx_mock = Mock()
-                redis_client_mock = Mock(name='redis_client_mock')
-                redis_ctx_mock.__exit__ = Mock()
-                redis_ctx_mock.__enter__ = Mock(return_value=redis_client_mock)
-                chan1.conn_or_acquire.return_value = redis_ctx_mock
-                qos = redis.QoS(chan1)
-                qos.visibility_timeout = timeout
-                qos.restore_visible()
-                redis_client_mock.zrevrangebyscore\
-                    .assert_called_with(chan1.unacked_index_key, timeout, 0,
-                                        start=0, num=10, withscores=True)
-
     def test_handle_event(self):
         p = self.Poller()
         chan = Mock(name='chan')
@@ -1349,7 +1394,6 @@ class test_MultiChannelPoller:
         channel._poll_error.assert_called_with('BRPOP')
 
 
-@skip.unless_module('redis')
 class test_Mutex:
 
     def test_mutex(self, lock_id='xxx'):
@@ -1390,7 +1434,6 @@ class test_Mutex:
         assert held
 
 
-@skip.unless_module('redis.sentinel')
 class test_RedisSentinel:
 
     def test_method_called(self):
@@ -1410,6 +1453,7 @@ class test_RedisSentinel:
     def test_getting_master_from_sentinel(self):
         with patch('redis.sentinel.Sentinel') as patched:
             connection = Connection(
+                'sentinel://localhost/;'
                 'sentinel://localhost:65532/;'
                 'sentinel://user@localhost:65533/;'
                 'sentinel://:password@localhost:65534/;'
@@ -1422,16 +1466,17 @@ class test_RedisSentinel:
             connection.channel()
             patched.assert_called_once_with(
                 [
+                    ('localhost', 26379),
                     ('localhost', 65532),
                     ('localhost', 65533),
                     ('localhost', 65534),
                     ('localhost', 65535),
                 ],
-                connection_class=mock.ANY, db=0, max_connections=10,
+                connection_class=ANY, db=0, max_connections=10,
                 min_other_sentinels=0, password=None, sentinel_kwargs=None,
                 socket_connect_timeout=None, socket_keepalive=None,
                 socket_keepalive_options=None, socket_timeout=None,
-                retry_on_timeout=None)
+                username=None, retry_on_timeout=None)
 
             master_for = patched.return_value.master_for
             master_for.assert_called()
@@ -1450,11 +1495,11 @@ class test_RedisSentinel:
             connection.channel()
             patched.assert_called_once_with(
                 [('localhost', 65532)],
-                connection_class=mock.ANY, db=0, max_connections=10,
+                connection_class=ANY, db=0, max_connections=10,
                 min_other_sentinels=0, password=None, sentinel_kwargs=None,
                 socket_connect_timeout=None, socket_keepalive=None,
                 socket_keepalive_options=None, socket_timeout=None,
-                retry_on_timeout=None)
+                username=None, retry_on_timeout=None)
 
             master_for = patched.return_value.master_for
             master_for.assert_called()
@@ -1472,3 +1517,99 @@ class test_RedisSentinel:
         )
         with pytest.raises(ConnectionError):
             connection.channel()
+
+    def test_missing_master_name_transport_option(self):
+        connection = Connection(
+            'sentinel://localhost:65534/',
+        )
+        with patch('redis.sentinel.Sentinel'),  \
+             pytest.raises(ValueError) as excinfo:
+            connection.connect()
+        expected = "'master_name' transport option must be specified."
+        assert expected == excinfo.value.args[0]
+
+    def test_sentinel_with_ssl(self):
+        ssl_params = {
+            'ssl_cert_reqs': 2,
+            'ssl_ca_certs': '/foo/ca.pem',
+            'ssl_certfile': '/foo/cert.crt',
+            'ssl_keyfile': '/foo/pkey.key'
+        }
+        with patch('redis.sentinel.Sentinel'):
+            with Connection(
+                    'sentinel://',
+                    transport_options={'master_name': 'not_important'},
+                    ssl=ssl_params) as conn:
+                params = conn.default_channel._connparams()
+                assert params['ssl_cert_reqs'] == ssl_params['ssl_cert_reqs']
+                assert params['ssl_ca_certs'] == ssl_params['ssl_ca_certs']
+                assert params['ssl_certfile'] == ssl_params['ssl_certfile']
+                assert params['ssl_keyfile'] == ssl_params['ssl_keyfile']
+                assert params.get('ssl') is None
+                from kombu.transport.redis import SentinelManagedSSLConnection
+                assert (params['connection_class'] is
+                        SentinelManagedSSLConnection)
+
+
+class test_GlobalKeyPrefixMixin:
+
+    from kombu.transport.redis import GlobalKeyPrefixMixin
+
+    global_keyprefix = "prefix_"
+    mixin = GlobalKeyPrefixMixin()
+    mixin.global_keyprefix = global_keyprefix
+
+    def test_prefix_simple_args(self):
+        for command in self.mixin.PREFIXED_SIMPLE_COMMANDS:
+            prefixed_args = self.mixin._prefix_args([command, "fake_key"])
+            assert prefixed_args == [
+                command,
+                f"{self.global_keyprefix}fake_key"
+            ]
+
+    def test_prefix_delete_args(self):
+        prefixed_args = self.mixin._prefix_args([
+            "DEL",
+            "fake_key",
+            "fake_key2",
+            "fake_key3"
+        ])
+
+        assert prefixed_args == [
+            "DEL",
+            f"{self.global_keyprefix}fake_key",
+            f"{self.global_keyprefix}fake_key2",
+            f"{self.global_keyprefix}fake_key3",
+        ]
+
+    def test_prefix_brpop_args(self):
+        prefixed_args = self.mixin._prefix_args([
+            "BRPOP",
+            "fake_key",
+            "fake_key2",
+            "not_prefixed"
+        ])
+
+        assert prefixed_args == [
+            "BRPOP",
+            f"{self.global_keyprefix}fake_key",
+            f"{self.global_keyprefix}fake_key2",
+            "not_prefixed",
+        ]
+
+    def test_prefix_evalsha_args(self):
+        prefixed_args = self.mixin._prefix_args([
+            "EVALSHA",
+            "not_prefixed",
+            "not_prefixed",
+            "fake_key",
+            "not_prefixed",
+        ])
+
+        assert prefixed_args == [
+            "EVALSHA",
+            "not_prefixed",
+            "not_prefixed",
+            f"{self.global_keyprefix}fake_key",
+            "not_prefixed",
+        ]
